INPUT: EQU $5CB0

;
; Mueve un tercio de una pantalla almacenada en memoria al tercio especificado.
;
; Entrada: HL -> Dirección de la pantalla almacenada en memoria.
;          B  -> Tercio de la pantalla almacenada en memoria (1, 2 o 3).
;          C  -> Tercio de la VideoRam sobre el que se vuelca (1, 2, 3).
;
; Altera el valor de los registros AF, BC, DE y HL.
;
; Tamaño: 86 bytes
;
MoveThird:
; Carga los valores
ld   HL, (INPUT)           ; HL = dirección dónde están los parámetros
ld   e, (HL)               ; E = parte baja dirección dónde está la pantalla
inc  hl
ld   d, (HL)               ; B = parte baja dirección dónde está la pantalla
inc  hl
ld   b, (hl)                ; B = tercio que se mueve de la pantalla almacenada
inc  hl
ld   c, (hl)                ; C = tercio de la VideoRAM sobre el que se vuelca
ex   de, hl                 ; HL = dirección dónde está la pantalla                            
; Calcula la dirección origen de los píxeles
ld   a, b                   ; A = tercio origen
ld   de, $0000              ; DE = desplazamiento tercio 1
dec  a                  
jr   z, moveThird_desPx     ; Si tercio = 1, salta
ld   d, $08                 ; DE = desplazamiento tercio 2
dec  a
jr   z, moveThird_desPx     ; Si tercio = 2, salta
ld   d, $10                 ; DE = desplazamiento tercio 3
moveThird_desPx:
or   a                      ; Pone acarreo a 0
adc  hl, de                 ; HL = dirección del tercio origen
; Calcula la dirección de destino de los píxeles
ld   a, c                   ; A = tercio destino
ld   de, $4000              ; DE = dirección tercio 1 VideoRAM
dec  a
jr   z, moveThird_px        ; Si tercio = 1, salta
ld   d, $48                 ; DE = dirección tercio 2 VideoRAM
dec  a
jr   z, moveThird_px        ; Si tercio = 2, salta
ld   d, $50                 ; DE = dirección tercio 3 VideoRAM
moveThird_px:
; Vuelca los píxeles del origen al destino
push bc
push hl                     ; Perserva registros
ld   bc, $800               ; BC = longitud píxeles tercio
ldir                        ; Vuelca origen en VideoRAM
pop  hl
pop  bc
; Calcula la dirección de origen de los atributos
ld   a, b                   ; A = tercio origen
ld   de, $1800              ; DE = desplazamiento atributos tercio 1
dec  a
jr   z, moveThird_desAttr   ; Si tercio = 1, salta
ld   d, $11                 ; DE = desplazamiento atributos tercio 2
dec  a
jr   z, moveThird_desAttr   ; Si tercio = 2, salta
ld   d, $0a                 ; DE = desplazamiento atributos tercio 3
moveThird_desAttr:
or   a                      ; Acarreo = 0
adc  hl, de                 ; HL = dirección atributos origen
; Calcula la dirección de destino de los atributos
ld   de, $5800              ; DE = dirección atributos tercio 1 VideoRAM
ld   a, c                   ; A = tercio destino
dec  a
jr   z, moveThird_attr      ; Si tercio = 1, salta
inc  d                      ; DE = dirección atributos tercio 2 VideoRAM
dec  a
jr   z, moveThird_attr      ; Si tercio = 2, salta
inc  d                      ; DE = dirección atributos tercio 3 VideoRAM
moveThird_attr:
; Vuelca los atributos del origen al destino
ld   bc, $100               ; BC = longitud atributos tercio
ldir                        ; Vuelca origen en VideoRAM
ret